<?php

require_once('images/Cube.php');
error_reporting(E_ALL & ~E_NOTICE);

/*
  Plugin Name: Rubik's Cube
  Plugin URI: http://www.vanderblonk.com/tools/wordpress-rubik-plugin/
  Description: A filter for WordPress that displays animated Rubik's cube algorithms
  Version: 2.1
  Author: Michiel van der Blonk
  Author URI: http://www.vanderblonk.com
 */

/* replace [rubik alg=... type=... width=... height=...]
 * by either
 * <canvas> as default type, using WebGL to draw an interactive cube
 * <iframe ...> when type=iframe, results in an iframe loaded from this domain
 * <object ...> when type=applet, results in a RubikPlayer applet taken from the local installed applet
 * <img ...> when type=img, results in an ImageCube png image
 * <a href=...> when type=link or popup, will open in new tab or in ThickBox
 */

function debugVar($s) {
    echo '<textarea rows="10" cols="40">';
    print_r($s);
    echo '</textarea><br />';
    return;
}

class RubikPlugin {

    var $regexp;
    var $templates;
    var $params; // not used yet
    var $templateNames;
    var $cube;

    /**
     * Constructor: returns the object with some variables preset
     * @access   public
     * @return   object
     */
    function RubikPlugin() {
        // other stuff
        $this->regexp = "/\[rubik [^\]]+\]/";

        $this->validAttributes = array(
            /* default */ 'width', 'height', 'type', 'alg',
            /* applet */ 'code', 'archive', 'scriptLanguage', 'displayLines', 'colorTable', 'scriptType', 'bgcolor', 'faces', 'initscript', 'stickerList', 'mode',
            'codebase_lookup', 'classloader_cache', 'java_arguments', 'image', 'boxborder', 'centerimage', 'permissions',
            /* animcube */ 'initrevmove', 'position', 'movetext', 'metric', 'fonttype', 'speed', 'doublespeed', 'perspective', 'scale', 'align', 'hint', 'edit', 'buttonbar', 'colorscheme',
            /* image */ 'stickers', 'format', 'id',
            /* link */ 'inline', 'linktext',
            /* webgl */ 'stickers', 'initscript'
        );
        $this->templateNames = array("iframe", "applet", "image", "link", "popup", "transformer", "transformer-with-close",
            "applet-animcube", "applet-caesar", "applet-rubikplayer", "applet-revenge", "applet-professor", "object", "visualcube",
            "webgl");

        foreach ($this->templateNames as $name)
            $this->templates[$name] = $this->getFileContents(dirname(__FILE__) . '/templates/' . $name . '.html');
        $this->cube = new Cube();
    }

    /**
     * Convert HarrisENG notation to SupersetENG notation
     * Since RubikPlayer 7 HarrisENG is not supported anymore.
     * but we need the new player because older versions are blocked by java security
     * @link http://www.randelshofer.ch/cubetwister/doc/
     */
    function convertHarrisENGtoSupersetENG($alg) {
        // repetition - not necessary
        $alg = str_replace("*", "", $alg);
        // cube rotations
        $alg = str_replace("x", "CR", $alg);
        $alg = str_replace("y", "CU", $alg);
        $alg = str_replace("z", "CF", $alg);
        // two layer moves
        $alg = str_replace("l", "TL", $alg);
        $alg = str_replace("r", "TR", $alg);
        $alg = str_replace("u", "TU", $alg);
        $alg = str_replace("d", "TD", $alg);
        $alg = str_replace("b", "TB", $alg);
        $alg = str_replace("f", "TF", $alg);
        // slice moves
        $alg = str_replace("M", "#", $alg); // temp char
        $alg = str_replace("S", "MF", $alg);
        $alg = str_replace("E", "MD", $alg);
        $alg = str_replace("#", "ML", $alg);
        $alg = str_replace("s", "SF", $alg);
        $alg = str_replace("m", "SL", $alg);
        $alg = str_replace("e", "SD", $alg);
        return $alg;
    }

    /**
     * @TODO: put this in some config file, like XML or ini format
     * @return \StdClass
     */
    function createTag($code) {
        $tag = new StdClass();
        $tag->tagsrc = $code;
        // defaults
        $tag->type = 'webgl';
        $tag->width = '250';
        $tag->height = '250';
        $tag->alg = '';
        $tag->scriptLanguage = 'HarrisENG';
        $tag->displayLines = 1;
        $tag->colorTable = 'b=0x003373,o=0xff4600,w=0xf8f8f8,g=0x00732f,r=0x8c000f,y=0xffd200,x=0x707070';
        $tag->scriptType = 'Solver';
        $tag->bgcolor = '0xFFFFFF';
        // $tag->faces = 'o,b,w,r,g,y';
        $tag->faces = '0,1,2,3,4,5';
        $tag->code = 'RubikPlayerFlat';
        $tag->archive = '/wp-content/plugins/rubik/applets/RubikPlayerFlat.jar';
        $tag->stickerList = '333333333444444444222222222000000000111111111555555555';
        $tag->format = 'png';
        // animcube
        $tag->position = '';
        $tag->movetext = '0';
        $tag->initrevmove = '';
        $tag->metric = 'f';
        $tag->fonttype = '1';
        $tag->speed = '10';
        $tag->doublespeed = '10';
        $tag->perspective = '2'; // normal
        $tag->scale = '0'; // fit
        $tag->align = '1'; // center
        $tag->hint = '';
        $tag->edit = '1';
        $tag->buttonbar = '1';
        $tag->colorscheme = "WYORGB"; // UDFBLR
        // popup
        $tag->linktext = "link";
        return $tag;
    }

    function fillAttributes($code, &$tag) {
        $paramOut = array();
        foreach ($this->validAttributes as $param) {
            preg_match("/${param}=\"[^\"]+\"/", $code, $paramOut);

            // check if we have a match and then read attributes
            if (!empty($paramOut[0])) {
                // split attribute name and value
                $attr = explode('=', $paramOut[0]);
                // check if we have a match and store in current tag object
                if (!empty($attr[0])) {
                    $tag->$attr[0] = str_replace('"', '', $attr[1]);
                }
            }
        }
        return $tag;
    }

    // for backwards compatibility with older applets or image software
    function fixOldStyleAttributes(&$tag) {
        // fix for older style type='generator' and type='solver'
        if (strtolower($tag->type) == 'generator' ||
                strtolower($tag->type) == 'solver')
            $tag->type = 'applet';

        // fix for older style type='stickers' instead of 'mode'
        if (strtolower($tag->stickers) == 'cross' ||
                strtolower($tag->stickers) == 'f2l' ||
                strtolower($tag->stickers) == 'll' ||
                strtolower($tag->stickers) == 'fl')
            $tag->mode = $tag->stickers;

        // convert HarrisENG to SupersetENG (for RubikPlayer 7)
        if ($tag->type == 'applet' || $tag->type == 'transformer' || $tag->type == 'transformer-with-close') {
            $tag->initscript = $this->convertHarrisENGtoSupersetENG($tag->initscript);
            $tag->alg = $this->convertHarrisENGtoSupersetENG($tag->alg);
        }
    }

    // note that when GD library is not valid
    // no image can be generated, and an error will be shown
    function createImageCubeStickers($tag) {
        $cube = $this->cube;
        $cube->reset();
        // correct jpg to jpeg
        if ($tag->format == 'jpg')
            $tag->format = 'jpeg';
        // first check if image creation is being supported
        $ok = function_exists('imagecreate');
        $ok &= function_exists('imageCreateTrueColor');
        $ok &= function_exists('imageCopyResampled');
        $ok &= function_exists('imageColorAllocate');
        $ok &= function_exists('imageColorTransparent');
        $ok &= function_exists('imageFill');
        $ok &= function_exists('image' . strtolower($tag->format));
        if (!$ok)
            $this->showError("warning: GD image library not configured correctly.");
        else {
            $cube->performAlg($tag->alg);
        }
        return $cube->toImageCubeStickers();
    }

    function createStickerList($tag) {
        $cube = $this->cube;
        if ($tag->type == 'image' || $tag->type == 'transformer' || $tag->type == 'transformer-with-close') {
            $tag->stickers = $this->createImageCubeStickers($tag);
        }

        if (preg_match('/applet/', $tag->type)) {
            if ($tag->mode)
                $cube->setMode($tag->mode);
            $tag->stickerList = $cube->toAppletStickers(false, ',');
        }

        if (preg_match('/webgl/', $tag->type)) {
            if ($tag->mode)
		$tag->stickers = $tag->mode;
        }

    }

    /**
     * Replace all tags matching the [rubik ] pattern by their corresponding html
     * @param   string	$content	The actual article or comment content
     * @access   public
     * @return   string	the content with all [rubik ..] tags replaced by proper html
     */
    function parse($content) {
        $tags = array();
        $rubikAlgs = array();
        // get rubik tags
        preg_match_all($this->regexp, $content, $rubikAlgs);
        $match = $rubikAlgs[0];
        // get attributes for each algorithm,
        // and store in a tag object (one for each code)
        foreach ($match as $code) {
            $tag = $this->createTag($code);
            $this->fillAttributes($code, $tag);
            $this->createStickerList($tag);
            $this->fixOldStyleAttributes($tag);

            if ($tag->type == 'applet-revenge' || $tag->type == 'applet-professor')
                $tag->scriptLanguage = "SupersetENG";
            if ($tag->type == 'link') {
                if (!$tag->inline)
                    $tag->inline = "false";
            }
            if ($tag->type == 'link' || $tag->type == 'popup')
                if ($tag->linktext == 'link')
                    $tag->linktext = $tag->alg;

	    if ($tag->type == 'webgl') {
                if ($tag->height > $tag->width)
                    $tag->height = $tag->width;
            }

            $html = $this->templates[$tag->type];
            // replace attributes in template
            foreach ($this->validAttributes as $name)
                $html = str_replace("@${name}", $tag->$name, $html);
            $tag->html = $html;
            // add to list of Tag objects
            $tags[] = $tag;
        }
        // start replacing templates in original content
        $out = $content;
        foreach ($tags as $tag)
            $out = str_replace($tag->tagsrc, $tag->html, $out);
        return $out;
    }

    function getFileContents($s) {
        if (file_exists($s))
            return file_get_contents($s);
    }

    function showError($s) {
        echo '<p style="color: red" class="plugin-error">' . $s . '</p>';
    }

    function testPlugin() {
        $s = 'This is a text without an algorithm';
        echo rubik_plugin($s) . "\n";
        $s = 'This is a text with one algorithm [rubik type="applet" alg="RL" width="340"] and some more text.';
        echo rubik_plugin($s) . "\n";
        $s = 'This is a text with two algorithms [rubik type="applet" alg="RL" width="340"] and some more text [rubik type="applet" alg="RL" width="340"]  and some more text';
        echo rubik_plugin($s) . "\n";
        $s = 'This is a text with two algorithms [rubik type="applet" alg="RL" width="340"] of different type [rubik type="image" alg="RL" width="340"]  and some more text';
        echo rubik_plugin($s) . "\n";
        $s = 'This is a text with two algorithms without optional arguments [rubik alg="RL"] and [rubik alg="R\'U\'RU" width="340"]  and some more text';
        echo rubik_plugin($s) . "\n";
        $s = 'This is a text with an AnimCube [rubik alg="RL" type="applet-animcube"]';
        echo rubik_plugin($s) . "\n";
        $s = 'This is a text with a Revenge [rubik alg="RL" type="applet-revenge"]';
        echo rubik_plugin($s) . "\n";
    }

}

function rubik_plugin($content) {
    $p = new RubikPlugin;
    $output = $p->parse($content);
    return $output;
}

function insert_webgl() {
    // styles
    wp_enqueue_style('bootstrap', 'http://maxcdn.bootstrapcdn.com/bootstrap/3.1.1/css/bootstrap.min.css');
    wp_enqueue_style('main', plugins_url('css/main.css', __FILE__));
    // scripts
    wp_enqueue_script("jquery-effects-core");
    wp_enqueue_script('requestanimationframe', plugins_url('js/requestanimationframe-fix.js', __FILE__));
    wp_enqueue_script('bootstrap', 'http://netdna.bootstrapcdn.com/bootstrap/3.1.1/js/bootstrap.min.js');
    wp_enqueue_script('webglmatrix', plugins_url('js/gl-matrix-min.js', __FILE__));
    wp_enqueue_script('rubiksmodels', plugins_url('js/models.js', __FILE__));
    wp_enqueue_script('rubikscube', plugins_url('js/rubiks.js', __FILE__), array('jquery', 'rubiksmodels')); 
    // webgl code
	echo file_get_contents(realpath(dirname(__FILE__)) . '/webgl-head.html');
}

// making sure this works outside of WordPress
if (function_exists('remove_filter'))
    remove_filter('the_content', 'wptexturize'); // disable smart quotes, these messup the code

if (function_exists('add_filter')) {
    add_filter('the_content', 'rubik_plugin');
    add_filter('comment_text', 'rubik_plugin');
}

if (function_exists('add_action')) {
    add_action('init', 'init_theme_method');
    if (!function_exists('init_theme_method')) {

        function init_theme_method() {
            add_thickbox();
        }

    }

	add_action('wp_head', 'insert_webgl');
}
?>
